<?php

namespace Litepie\Role\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Role\Events\PermissionAction as PermissionActionEvent;
use Litepie\Role\Notifications\PermissionAction as PermissionActionNotification;
use Litepie\Actions\Concerns\AsAction;

class PermissionAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the PermissionActionEvent.
     *
     * @param   PermissionActionEvent  $event
     * @return mixed
     */
    public function handle(PermissionActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $permission.
     *
     * @param   PermissionActionEvent  $event
     * @return void
     */
    public function create(PermissionActionEvent $event)
    {
        $client = $event->permission->client;
        Notification::send($client, new PermissionActionNotification($event));
    }

    /**
     * Handle the PermissionActionEvent as a listener.
     *
     * @param   PermissionActionEvent  $event
     * @return mixed
     */
    public function asListener(PermissionActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   PermissionActionEvent  $event
     * @return bool
     */
    private function isAllowed(PermissionActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
