<?php

namespace Litepie\Role\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Role\Events\PermissionWorkflow as PermissionWorkflowEvent;
use Litepie\Role\Notifications\PermissionWorkflow as PermissionWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class PermissionWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(PermissionWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $permission is submitted.
     *
     * @param PermissionWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(PermissionWorkflowEvent $event)
    {
        $client = $event->permission->client;
        Notification::send($client, new PermissionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $permission is published.
     *
     * @param PermissionWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(PermissionWorkflowEvent $event)
    {
        $client = $event->permission->client;
        Notification::send($client, new PermissionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $permission is approved.
     *
     * @param PermissionWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(PermissionWorkflowEvent $event)
    {
        $client = $event->permission->client;
        Notification::send($client, new PermissionWorkflowNotification($event));
    }

    /**
     * Handles the $permission workflow event as a listener.
     *
     * @param PermissionWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(PermissionWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
